// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#ifdef GL_FRAGMENT_PRECISION_HIGH
    #define hmp highp
#else
    #define hmp mediump
#endif

#include "fragmentVersionSimple.h"
#include "simplexNoise.glsl"

varying float fog;
varying mediump vec3 worldPos;

uniform vec4 FOG_COLOR;
uniform vec4 CURRENT_COLOR;
uniform vec2 FOG_CONTROL;
uniform hmp float TOTAL_REAL_WORLD_TIME;

struct Fancy_Misty_Sky_Colour {
    vec3 daySky;
    vec3 duskOrDawnSky;
    vec3 nightSky;
    vec3 dayScatter;
    vec3 duskOrDawnScatter;
    vec3 nightScatter;
    vec3 dayCloud;
    vec3 duskOrDawnCloud;
    vec3 nightCloud;
};

struct Fancy_Misty_Data {
    float scatter;
    float daylight;
    float rain;
    float duskOrDawn;
};

void main() {
Fancy_Misty_Sky_Colour colour = Fancy_Misty_Sky_Colour(
    vec3(0.40, 0.85, 1.00), // Day sky
    vec3(0.6,  0.8,  0.9),  // Dusk or dawn sky
    vec3(0.00, 0.15, 0.2),  // Night sky
    vec3(1.0,  0.8,  0.5),  // Day scatter
    vec3(0.7,  0.4,  0.2),  // Dusk or dawn scatter
    vec3(0.15, 0.2,  0.25), // Night scatter
    vec3(1.0,  0.98, 0.97), // Day cloud
    vec3(1.0,  0.8,  0.1),  // Dusk or dawn cloud
    vec3(0.1,  0.3,  0.45)  // night cloud
);

Fancy_Misty_Data data = Fancy_Misty_Data(
    smoothstep(0.00, 0.85, fog),             // Scatter
    clamp(FOG_COLOR.g * 1.5, 0.0, 1.0) *
        clamp(FOG_COLOR.g * 1.5, 0.0, 1.0),          // Daylight
    smoothstep(0.690, 0.300, FOG_CONTROL.x),         // Rain
    clamp(FOG_COLOR.r - FOG_COLOR.b, 0.0, 0.5) * 2.0 // Dusk or dawn
);

vec3 diffuse = CURRENT_COLOR.rgb;

diffuse = mix(diffuse, mix(colour.daySky, colour.dayScatter, data.scatter), data.daylight);                 // Day sky
diffuse = mix(diffuse, mix(colour.nightSky, colour.nightScatter, data.scatter), 1.0 - data.daylight);       // Night sky
diffuse = mix(diffuse, mix(colour.duskOrDawnSky, colour.duskOrDawnScatter, data.scatter), data.duskOrDawn); // Dusk or dawn sky
diffuse = mix(diffuse, vec3(dot(diffuse, vec3(0.2126, 0.7152, 0.0722))), data.rain);                             // Rainy sky

vec3 cloudColour = mix(vec3(1.0), colour.dayCloud, data.daylight);               // Day cloud
    cloudColour = mix(cloudColour, colour.duskOrDawnCloud, data.duskOrDawn); // Dusk or dawn cloud
    cloudColour = mix(cloudColour, colour.nightCloud, 1.0 - data.daylight);  // Night cloud
    cloudColour = mix(cloudColour, 
        vec3(dot(cloudColour, vec3(0.2126, 0.7152, 0.0722))), data.rain);         // Rainy cloud
float cloudShape = fBM(worldPos.xz * 1.20, 0.525, 0.800, 4, TOTAL_REAL_WORLD_TIME);

if (cloudShape > 0.0) {
    cloudColour *= mix(0.8, 1.0, fBM(worldPos.xz * 1.18, 0.5, 1.0, 3, TOTAL_REAL_WORLD_TIME));
}

diffuse = mix(diffuse, cloudColour, cloudShape);

    gl_FragColor = mix(vec4(diffuse, 1.0), FOG_COLOR, fog);

}