#include "ShaderConstants.fxh"
#include "simplexNoise.hlsl"

struct PS_Input {
    float4 position : SV_Position;
    float fog : fog;
    float3 worldPos : worldPos;
};

struct PS_Output {
    float4 color : SV_Target;
};

struct Fancy_Misty_Sky_Colour {
    float3 daySky;
    float3 duskOrDawnSky;
    float3 nightSky;
    float3 dayScatter;
    float3 duskOrDawnScatter;
    float3 nightScatter;
    float3 dayCloud;
    float3 duskOrDawnCloud;
    float3 nightCloud;
};

struct Fancy_Misty_Data {
    float scatter;
    float daylight;
    float rain;
    float duskOrDawn;
};

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput) {
struct Fancy_Misty_Sky_Colour colour = {
    {
        float3(0.40, 0.85, 1.00), // Day sky
        float3(0.6,  0.8,  0.9),  // Dusk or dawn sky
        float3(0.00, 0.15, 0.2),  // Night sky
        float3(1.0,  0.8,  0.5),  // Day scatter
        float3(0.7,  0.4,  0.2),  // Dusk or dawn scatter
        float3(0.15, 0.2,  0.25), // Night scatter
        float3(1.0,  0.98, 0.97), // Day cloud
        float3(1.0,  0.8,  0.1),  // Dusk or dawn cloud
        float3(0.1,  0.3,  0.45)  // night cloud
    }
};

struct Fancy_Misty_Data data = {
    {
        smoothstep(0.00, 0.85, PSInput.fog),             // Scatter
        clamp(FOG_COLOR.g * 1.5, 0.0, 1.0) *
            clamp(FOG_COLOR.g * 1.5, 0.0, 1.0),          // Daylight
        smoothstep(0.690, 0.300, FOG_CONTROL.x),         // Rain
        clamp(FOG_COLOR.r - FOG_COLOR.b, 0.0, 0.5) * 2.0 // Dusk or dawn
    }
};

float3 diffuse = CURRENT_COLOR;

diffuse = lerp(diffuse, lerp(colour.daySky, colour.dayScatter, data.scatter), data.daylight);                 // Day sky
diffuse = lerp(diffuse, lerp(colour.nightSky, colour.nightScatter, data.scatter), 1.0 - data.daylight);       // Night sky
diffuse = lerp(diffuse, lerp(colour.duskOrDawnSky, colour.duskOrDawnScatter, data.scatter), data.duskOrDawn); // Dusk or dawn sky
diffuse = lerp(diffuse, dot(diffuse, float3(0.2126, 0.7152, 0.0722)), data.rain);                             // Rainy sky

float3 cloudColour = lerp(1.0, colour.dayCloud, data.daylight);               // Day cloud
    cloudColour = lerp(cloudColour, colour.duskOrDawnCloud, data.duskOrDawn); // Dusk or dawn cloud
    cloudColour = lerp(cloudColour, colour.nightCloud, 1.0 - data.daylight);  // Night cloud
    cloudColour = lerp(cloudColour, 
        dot(cloudColour, float3(0.2126, 0.7152, 0.0722)), data.rain);         // Rainy cloud
float cloudShape = fBM(PSInput.worldPos.xz * 1.20, 0.525, 0.800, 4, TOTAL_REAL_WORLD_TIME);

if (cloudShape > 0.0) {
    cloudColour *= lerp(0.8, 1.0, fBM(PSInput.worldPos.xz * 1.18, 0.5, 1.0, 3, TOTAL_REAL_WORLD_TIME));
}

diffuse = lerp(diffuse, cloudColour, cloudShape);

    PSOutput.color = lerp(float4(diffuse, 1.0), FOG_COLOR, PSInput.fog);
}