#include "ShaderConstants.fxh"

struct VS_Input {
    float3 position : POSITION;
    float color : COLOR;
#ifdef INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
};

struct PS_Input {
    float4 position : SV_Position;
    float fog : fog;
    float3 worldPos : worldPos;
#ifdef GEOMETRY_INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	uint renTarget_id : SV_RenderTargetArrayIndex;
#endif
};

ROOT_SIGNATURE
void main(in VS_Input VSInput, out PS_Input PSInput) {
float4 planePos = float4(VSInput.position, 1);
planePos.y -= length(planePos.xz) * 0.15;
#ifdef INSTANCEDSTEREO
	int i = VSInput.instanceID;
	PSInput.position = mul(WORLDVIEWPROJ_STEREO[i], planePos);
#else
	PSInput.position = mul(WORLDVIEWPROJ, planePos);
#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
	PSInput.instanceID = VSInput.instanceID;
#endif 
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	PSInput.renTarget_id = VSInput.instanceID;
#endif
    PSInput.fog = VSInput.color.r;
	PSInput.worldPos = VSInput.position;
}